/* Load picture with datatypes and save as PPM; needs picture.datatype V43+   */

/*----------------------------------------------------------------------------*/
/* System Includes                                                            */
/*----------------------------------------------------------------------------*/

	#include <proto/exec.h>
	#include <proto/dos.h>
	#include <proto/utility.h>
	#include <clib/alib_protos.h>
	#include <proto/datatypes.h>
	#include <datatypes/pictureclass.h>

/*----------------------------------------------------------------------------*/
/* Type definitions                                                           */
/*----------------------------------------------------------------------------*/

	struct rgbarray {
		UWORD w;
		UWORD h;
		UBYTE array[2];
		};

/*----------------------------------------------------------------------------*/
/* Allocate new RGB array                                                     */
/*----------------------------------------------------------------------------*/

	struct rgbarray *alloc_rgbarray (long w,long h)

	{
	struct rgbarray *a;

	if (a = AllocVec (sizeof(struct rgbarray) - 2 + w * h * 3,MEMF_CLEAR))
		{
		a->w = w;
		a->h = h;
		}

	return (a);
	}

/*----------------------------------------------------------------------------*/
/* Free RGB array                                                             */
/*----------------------------------------------------------------------------*/

	void free_rgbarray (struct rgbarray *a)

	{
	FreeVec (a);
	}

/*----------------------------------------------------------------------------*/
/* Check if datatype object supports the given method                         */
/*----------------------------------------------------------------------------*/

	BOOL check_dt_method (Object *dto,ULONG method)

	{
	ULONG *list;

	if (list = GetDTMethods (dto))
		{
		ULONG *m;

		for (m = list; *m != ~0; m++)
			if (*m == method)
				return (TRUE);
		}

	return (FALSE);
	}

/*----------------------------------------------------------------------------*/
/* Load picture into RGB array                                                */
/*----------------------------------------------------------------------------*/

	struct rgbarray *load_dt_picture (char *name)

	{
	struct rgbarray *a = NULL;
	Object *dto;

	if (dto = NewDTObject (name,
			DTA_GroupID,GID_PICTURE,
			PDTA_Remap,FALSE,
			PDTA_DestMode,PMODE_V43,
			TAG_END))
		{
		struct BitMapHeader *bmhd = NULL;

		GetDTAttrs (dto,
			PDTA_BitMapHeader,&bmhd,
			TAG_END);

		if (bmhd && bmhd->bmh_Depth > 8)
			{
			if (check_dt_method (dto,PDTM_READPIXELARRAY))
				{
				if (a = alloc_rgbarray (bmhd->bmh_Width,bmhd->bmh_Height))
					{
					DoMethod (dto,PDTM_READPIXELARRAY,a->array,PBPAFMT_RGB,a->w * 3,0,0,a->w,a->h);
					}
				else
					SetIoErr (ERROR_NO_FREE_STORE);
				}
			else
				SetIoErr (ERROR_ACTION_NOT_KNOWN);
			}
		else
			SetIoErr (ERROR_OBJECT_WRONG_TYPE);

		DisposeDTObject (dto);
		}

	return (a);
	}

/*----------------------------------------------------------------------------*/
/* Save RGB array as PPM file                                                 */
/*----------------------------------------------------------------------------*/

	BOOL save_ppm (struct rgbarray *a,char *name)

	{
	BOOL ok = FALSE;
	BPTR fh;

	if (fh = Open (name,MODE_NEWFILE))
		{
		FPrintf (fh,"P6\n%ld %ld\n255\n",a->w,a->h);
		FWrite (fh,a->array,a->w * a->h * 3,1);

		ok = TRUE;

		Close (fh);
		}

	return (ok);
	}

/*----------------------------------------------------------------------------*/
/* Main program                                                               */
/*----------------------------------------------------------------------------*/

	int main (void)

	{
	struct RDArgs *rdargs;
	static struct {
		char *from;
		char *to;
		} args;
	struct rgbarray *a;
	int rc = RETURN_FAIL;

	rdargs = ReadArgs ("FROM/A,TO/A",(APTR)&args,NULL);
	if (!rdargs)
		{
		PrintFault (IoErr(),NULL);
		return (RETURN_ERROR);
		}

	if (a = load_dt_picture (args.from))
		{
		if (save_ppm (a,args.to))
			rc = RETURN_OK;
		else
			PrintFault (IoErr(),args.to);

		free_rgbarray (a);
		}
	else
		PrintFault (IoErr(),args.from);

	return (rc);
	}

/*----------------------------------------------------------------------------*/
/* End of source code                                                         */
/*----------------------------------------------------------------------------*/
